pub struct MoveGenerator { /* private fields */ }Implementations
Sourceimpl MoveGenerator
impl MoveGenerator
pub fn get_single_legal_move( position: &ChessPosition, from_bitboard: BitBoard, to_bitboard: BitBoard, ) -> Option<Move>
pub fn has_legal_moves( position: &ChessPosition, from_bitboard: BitBoard, to_bitboard: BitBoard, ) -> bool
pub fn new_legal(position: &ChessPosition) -> Self
Sourcepub fn reorganize_square_and_bitboard_array(&mut self)
pub fn reorganize_square_and_bitboard_array(&mut self)
The iterator portion of this struct relies on the invariant that the bitboards at the beginning of the square_and_bitboard_array[] array are the only ones used. As a result, we must partition the list such that the assumption is true.
pub fn remove_mask(&mut self, mask: BitBoard)
pub fn remove_move(&mut self, move_: Move) -> bool
pub fn get_from_bitboard_iterator_mask(&self) -> BitBoard ⓘ
pub fn set_from_bitboard_iterator_mask(&mut self, mask: BitBoard)
pub fn get_to_bitboard_iterator_mask(&self) -> BitBoard ⓘ
pub fn set_to_bitboard_iterator_mask(&mut self, mask: BitBoard)
pub fn set_iterator_masks( &mut self, from_bitboard: BitBoard, to_bitboard: BitBoard, )
pub fn perft_test(position: &ChessPosition, depth: usize) -> usize
pub fn perft_test_piecewise(position: &ChessPosition, depth: usize) -> usize
pub fn iter(&self) -> impl Iterator<Item = Move>
pub fn len(&self) -> usize
pub fn is_empty(&self) -> bool
pub fn contains(&self, move_: &Move) -> bool
pub fn is_legal(position: &ChessPosition, move_: &Move) -> bool